/*
 * main.c
 *
 *  Created on: 2012-10-11
 *  USBRelay 8 przekanikw sterowanych przez USB z funkcj pomiaru temperatury w pomieszczeniu
 *       Autor: Piotr Rosenbaum
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "1Wire/ds18x20.h"
#include "UART/uart.h"

#define PK_1 (1<<PC0)															//definicja pinu do ktrego podczona jest przekanik
#define PK_ON_1 PORTC |= PK_1													//definicja wczania przekaznika (stan wysoki)
#define PK_OFF_1 PORTC &= ~ PK_1

#define PK_2 (1<<PC1)															//definicja pinu do ktrego podczona jest przekanik
#define PK_ON_2 PORTC |= PK_2													//definicja wczania przekaznika (stan wysoki)
#define PK_OFF_2 PORTC &= ~ PK_2

#define PK_3 (1<<PC2)															//definicja pinu do ktrego podczona jest przekanik
#define PK_ON_3 PORTC |= PK_3													//definicja wczania przekaznika (stan wysoki)
#define PK_OFF_3 PORTC &= ~ PK_3

#define PK_4 (1<<PC3)															//definicja pinu do ktrego podczona jest przekanik
#define PK_ON_4 PORTC |= PK_4													//definicja wczania przekaznika (stan wysoki)
#define PK_OFF_4 PORTC &= ~ PK_4

#define PK_5 (1<<PC4)															//definicja pinu do ktrego podczona jest przekanik
#define PK_ON_5 PORTC |= PK_5													//definicja wczania przekaznika (stan wysoki)
#define PK_OFF_5 PORTC &= ~ PK_5

#define PK_6 (1<<PC5)															//definicja pinu do ktrego podczona jest przekanik
#define PK_ON_6 PORTC |= PK_6													//definicja wczania przekaznika (stan wysoki)
#define PK_OFF_6 PORTC &= ~ PK_6

#define PK_7 (1<<PD2)															//definicja pinu do ktrego podczona jest przekanik
#define PK_ON_7 PORTD |= PK_7													//definicja wczania przekaznika (stan wysoki)
#define PK_OFF_7 PORTD &= ~ PK_7

#define PK_8 (1<<PD3)															//definicja pinu do ktrego podczona jest przekanik
#define PK_ON_8 PORTD |= PK_8													//definicja wczania przekaznika (stan wysoki)
#define PK_OFF_8 PORTD &= ~ PK_8

uint8_t subzero;																//Okreslenie znaku temperatury mierzonej
uint8_t cel;																	//Czesc calkowita temperatury mierzonej
uint8_t cel_fract_bits;															//Czesc ulamkowa temperatury mierzonej
uint8_t ilosc_czujnikow;
char komenda = 0;


int main (void)
{
	DDRC |= PK_1;											// kierunek pinu PC0  wyjciowy (1) (przekanik)
	DDRC |= PK_2;											// kierunek pinu PC1  wyjciowy (1) (przekanik)
	DDRC |= PK_3;											// kierunek pinu PC2  wyjciowy (1) (przekanik)
	DDRC |= PK_4;											// kierunek pinu PC3  wyjciowy (1) (przekanik)
	DDRC |= PK_5;											// kierunek pinu PC4  wyjciowy (1) (przekanik)
	DDRC |= PK_6;											// kierunek pinu PC5  wyjciowy (1) (przekanik)
	DDRD |= PK_7;											// kierunek pinu PD2  wyjciowy (1) (przekanik)
	DDRD |= PK_8;											// kierunek pinu PD3  wyjciowy (1) (przekanik)

	PK_OFF_1;
	PK_OFF_2;
	PK_OFF_3;
	PK_OFF_4;
	PK_OFF_5;												//wyczenie wszystkich przekanikw (stan niski)
	PK_OFF_6;
	PK_OFF_7;
	PK_OFF_8;

	TCCR1B  |= (1<<WGM12);                                   //CTC
	TCCR1B  |= (1<<CS12);
	TIMSK   |= (1<<OCIE1A);                                  // przerwanie Compare Match A

	OCR1A = 31250;											//ustawienie rejestru przepenienia, po to aby przerwanie pojawiao si co 1sek.

	USART_Init(__UBRR);
	sei();

	ilosc_czujnikow = search_sensors();																			//Zczytywanie dostepnych czujnikow na magistrali
	DS18X20_start_meas(DS18X20_POWER_EXTERN, NULL);																//Polecenie pomiaru dla czujnikw
	_delay_ms(600);																								//Oczekiwanie na konwersje
	if( DS18X20_OK == DS18X20_read_meas(gSensorIDs[0], &subzero, &cel, &cel_fract_bits) );



	while(1)
	{

		komenda = uart_getc();							//Odbir pojedynczego znaku z UART (komendy)


		switch(komenda)									//switch ktry decyduje co zrobi na podstawie konkretnej komendy otrzymanej z USB
		{
			case 'a':
				PK_ON_1;
			break;
			case 'b':
				PK_OFF_1;
			break;
			case 'c':
				PK_ON_2;
			break;
			case 'd':
				PK_OFF_2;
			break;
			case 'e':
				PK_ON_3;
			break;
			case 'f':
				PK_OFF_3;
			break;
			case 'g':
				PK_ON_4;
			break;
			case 'h':
				PK_OFF_4;
			break;
			case 'i':
				PK_ON_5;
			break;
			case 'j':
				PK_OFF_5;
			break;
			case 'k':
				PK_ON_6;
			break;
			case 'l':
				PK_OFF_6;
			break;
			case 'm':
				PK_ON_7;
			break;
			case 'n':
				PK_OFF_7;
			break;
			case 'o':
				PK_ON_8;
			break;
			case 'p':
				PK_OFF_8;
			break;

			case 'y':											//przypadek ten wcza wszystkie przekaniki
				PK_ON_1;
				PK_ON_2;
				PK_ON_3;
				PK_ON_4;
				PK_ON_5;
				PK_ON_6;
				PK_ON_7;
				PK_ON_8;
			break;
			case 'z':											//przypadek ten wycza wszystkie przekaniki
				PK_OFF_1;
				PK_OFF_2;
				PK_OFF_3;
				PK_OFF_4;
				PK_OFF_5;
				PK_OFF_6;
				PK_OFF_7;
				PK_OFF_8;
			break;
		}

		komenda = 0;											//czysczenie zmiennej po sprawdzeniu
		_delay_ms(300);
	}
}


ISR(TIMER1_COMPA_vect)                               //procedura przerwania TMIERA1 Compare Match A (przerwanie co 1sek.)
{

	DS18X20_start_meas(DS18X20_POWER_EXTERN, NULL);		//Polecenie pomiaru dla czujnikw
	_delay_ms(30);

	if( DS18X20_OK == DS18X20_read_meas(gSensorIDs[0], &subzero, &cel, &cel_fract_bits) )		//Odczyt pomiaru i sprawdzenie czy jest czujnik
	{
		_delay_ms(30);
		uart_putint(cel,10);					//wylij cz dziesitn temperatury
		uart_putc('.');							//wylij kropk pomidzy wartociami
		uart_putint(cel_fract_bits,10);			//wylij cz uamkow temperatury
		uart_puts("\r\n");						// wylij znak CR (enter) i LF (nowa linia)
		_delay_ms(300);
	}
	else										//gdy nie wykryto czujnika...
	{
		uart_puts("b.d.");						//wylij b.d - brak danych
		uart_puts("\r\n");						// wylij znak CR (enter) i LF (nowa linia)
		_delay_ms(300);
	}

}






